#![feature(async_closure)]

use std::{path::{Path, PathBuf}, time::Duration};
use xmpd_manifest::{store::JsonStore, Manifest};

#[macro_use]
mod macros;
mod main_window;
mod windows;
mod components;
mod data;
mod utils;

const W_NAME: &str = "xmpd v2.0.0a";

type Result<T> = anyhow::Result<T>;

pub fn start(manifest_path: PathBuf) -> Result<()> {
    let options = eframe::NativeOptions::default();
    let mut state = GuiState::new(&manifest_path)?;
    let theme = xmpd_settings::Settings::get()?.theme.clone(); 
    let res = eframe::run_simple_native(W_NAME, options, move |ctx, _frame| {
        egui_extras::install_image_loaders(ctx);
        state.windows.clone().draw_all(ctx, &mut state);
        egui::CentralPanel::default()
            .frame(
                egui::Frame::none()
                    .fill(theme.primary_bg_color)
                    .stroke(egui::Stroke::new(
                        1.0,
                        theme.secondary_bg_color,
                    )),
            )
            .show(ctx, |ui| {
                ui.style_mut().visuals.override_text_color = Some(theme.text_color);
                main_window::draw(ui, &mut state)
            });
        ctx.request_repaint_after(Duration::from_millis(500));
    });
    if let Err(e) = res { // dumb err value by eframe
        anyhow::bail!(e.to_string());
    }
    Ok(())
}


pub enum Message {
    
}

pub struct GuiState {
    pub manifest: Manifest<JsonStore>,
    pub windows: windows::Windows
}

impl GuiState {
    pub fn new(manifest_path: &Path) -> Result<Self> {
        Ok(Self {
            manifest: Manifest::new(manifest_path)?,
            windows: windows::Windows::new(),
        })
    }
}



